//
//  AppDelegate.swift
//  Spatial
//
//  Created by Phidgets on 2018-02-15.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import Cocoa
import Phidget22Swift

@NSApplicationMain
class AppDelegate: NSObject, NSApplicationDelegate {

    @IBOutlet weak var window: NSWindow!
    let ch = Spatial()
    @IBOutlet weak var phidgetInfoBox: PhidgetInfoBox!
    
    //GUI
    @IBOutlet weak var dataIntervalSlider: NSSlider!
    @IBOutlet weak var minDataIntervalLabel: NSTextField!
    @IBOutlet weak var maxDataIntervalLabel: NSTextField!
    @IBOutlet weak var dataIntervalLabel: NSTextField!
    @IBOutlet weak var settingsBox: NSBox!
    @IBOutlet weak var dataBox: NSBox!
    @IBOutlet weak var accelXLabel: NSTextField!
    @IBOutlet weak var accelYLabel: NSTextField!
    @IBOutlet weak var accelZLabel: NSTextField!
    @IBOutlet weak var gyroXLabel: NSTextField!
    @IBOutlet weak var gyroYLabel: NSTextField!
    @IBOutlet weak var gyroZLabel: NSTextField!
    @IBOutlet weak var magXLabel: NSTextField!
    @IBOutlet weak var magYLabel: NSTextField!
    @IBOutlet weak var magZLabel: NSTextField!
    @IBOutlet weak var timestampLabel: NSTextField!
    
    //Error Window
    @IBOutlet weak var errorEventWindow: NSPanel!
    @IBOutlet var errorEventLog: NSTextView!
    @IBOutlet weak var errorEventCount: NSTextField!
    var errorCounter = 0
    
    //MARK: View events
    func applicationDidFinishLaunching(_ aNotification: Notification) {
        do {
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.minSize.height
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
            
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.spatialData.addHandler(spatialdata_handler)
            
            /*
             * Please review the Phidget22 channel matching documentation for details on the device
             * and class architecture of Phidget22, and how channels are matched to device features.
             */
            
            /*
             * Specifies the serial number of the device to attach to.
             * For VINT devices, this is the hub serial number.
             *
             * The default is any device.
             */
            // try ch.setDeviceSerialNumber(<YOUR DEVICE SERIAL NUMBER>)
            
            /*
             * For VINT devices, this specifies the port the VINT device must be plugged into.
             *
             * The default is any port.
             */
            // try ch.setHubPort(0)
            
            /*
             * Specifies which channel to attach to.  It is important that the channel of
             * the device is the same class as the channel that is being opened.
             *
             * The default is any channel.
             */
            // try ch.setChannel(0)
            
            /*
             * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
             * In a normal environment this can be done automatically by enabling server discovery, which
             * will cause the client to discovery and connect to available servers.
             *
             * To force the channel to only match a network Phidget, set remote to 1.
             */
            
            // try Net.enableServerDiscovery(serverType: .deviceRemote)
            // try ch.setIsRemote(true)
            
            /*
             openCmdLine allows command line arguments to be passed to this program in order to
             configure channel matching.
             If you have modified the code and added your own channel matching, comment out
             the openCmdLine call, and simply replace it with the following line:
             */
            //try ch.open()
            
            
            try phidgetInfoBox.openCmdLine(phid: ch as Phidget)
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    func applicationShouldTerminateAfterLastWindowClosed(_ sender: NSApplication) -> Bool {
        return true
    }
    
    func applicationWillTerminate(_ aNotification: Notification) {
        //Safely close example
        do{
            print("Closing")
            ch.attach.removeAllHandlers()
            ch.detach.removeAllHandlers()
            ch.error.removeAllHandlers()
            ch.spatialData.removeAllHandlers()
            try ch.close()
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event handlers
    func attach_handler(sender: Phidget){
        do{
            let attachedDevice = sender as! Spatial

            let minDataInterval = try attachedDevice.getMinDataInterval()
            let maxDataInterval = try attachedDevice.getMaxDataInterval()
            let dataInterval = try attachedDevice.getDataInterval()
            
            DispatchQueue.main.async {
                self.phidgetInfoBox.fillPhidgetInfo(device: sender)
                self.dataBox.isHidden = false
                self.settingsBox.isHidden = false
                
                self.minDataIntervalLabel.stringValue = String(format:"%d",minDataInterval)
                self.maxDataIntervalLabel.stringValue = String(format:"%d",maxDataInterval)
                self.dataIntervalLabel.stringValue = String(format:"%d ms",dataInterval)
                
                self.dataIntervalSlider.minValue = Double(minDataInterval)
                self.dataIntervalSlider.maxValue = Double(maxDataInterval)
                self.dataIntervalSlider.doubleValue = Double(dataInterval)
                
                //Adjusting window height
                var frame = self.window.frame
                frame.size.height = self.window.maxSize.height
                self.window.setFrame(frame, display: true, animate: false)
                self.window.center()
            }
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    func detach_handler(sender: Phidget){
        DispatchQueue.main.async {
            self.phidgetInfoBox.fillPhidgetInfo(device: nil)
            self.settingsBox.isHidden = true
            self.dataBox.isHidden = true
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.minSize.height
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        outputError(errorDescription: data.description, errorCode: data.code.rawValue)
    }
    
    func spatialdata_handler(sender: Spatial, data: (acceleration: [Double], angularRate: [Double], magneticField: [Double], timestamp: Double)){
        DispatchQueue.main.async {
            
            self.accelXLabel.stringValue = String(format:"%.3f g", data.acceleration[0])
            self.accelYLabel.stringValue = String(format:"%.3f g", data.acceleration[1])
            self.accelZLabel.stringValue = String(format:"%.3f g", data.acceleration[2])
            
            self.gyroXLabel.stringValue = String(format:"%.3f °/s", data.angularRate[0])
            self.gyroYLabel.stringValue = String(format:"%.3f °/s", data.angularRate[1])
            self.gyroZLabel.stringValue = String(format:"%.3f °/s", data.angularRate[2])
            
            self.magXLabel.stringValue = String(format:"%.3f G", data.magneticField[0])
            self.magYLabel.stringValue = String(format:"%.3f G", data.magneticField[1])
            self.magZLabel.stringValue = String(format:"%.3f G", data.magneticField[2])
            
            self.timestampLabel.stringValue = String(format:"%.0f ms", data.timestamp)

        }
    }
    
    //MARK: GUI Controls
    @IBAction func setDataInterval(_ sender: Any) {
        let slider = sender as! NSSlider
        do{
            try ch.setDataInterval(UInt32(slider.intValue))
            dataIntervalLabel.stringValue = String(format:"%d ms", UInt32(slider.intValue))
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    //MARK: Error
    func outputError(errorDescription:String, errorCode:UInt32){
        DispatchQueue.main.async {
            self.errorCounter += 1
            let outputString = NSAttributedString(string: String(format:"Error %d: %@\n", errorCode, errorDescription))
            self.errorEventLog.textStorage?.beginEditing()
            self.errorEventLog.textStorage?.append(outputString)
            self.errorEventLog.textStorage?.endEditing()
            self.errorEventCount.stringValue = "\(self.errorCounter)"
            if(!self.errorEventWindow.isVisible){
                self.errorEventWindow.setIsVisible(true)
                self.errorEventWindow.center()
            }
        }
        
    }
    
    @IBAction func clearError(_ sender: Any) {
        errorEventLog.textStorage?.setAttributedString(NSAttributedString(string: ""))
        errorCounter = 0
        errorEventCount.stringValue = "0"
    }
}

